﻿// Copyright (c) Six Labors and contributors.
// Licensed under the Apache License, Version 2.0.

// <auto-generated />


using System;
using System.Numerics;
using System.Runtime.CompilerServices;

namespace SixLabors.ImageSharp.PixelFormats.PixelBlenders
{
    internal static partial class PorterDuffFunctions
    {





        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 NormalSrc(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return source;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 NormalSrcAtop(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Atop(backdrop, source, Normal(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 NormalSrcOver(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Over(backdrop, source, Normal(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 NormalSrcIn(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return In(backdrop, source, Normal(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 NormalSrcOut(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Out(backdrop, source);
        }

       [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 NormalDest(Vector4 backdrop, Vector4 source, float opacity)
        {
            return backdrop;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 NormalDestAtop(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Atop(source, backdrop, Normal(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 NormalDestOver(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Over(source, backdrop, Normal(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 NormalDestIn(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return In(source, backdrop, Normal(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 NormalDestOut(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Out(source, backdrop);
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 NormalXor(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Xor(backdrop, source);
        }   

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 NormalClear(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Clear(backdrop, source);
        }
        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel NormalSrc<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(NormalSrc(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel NormalSrcAtop<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(NormalSrcAtop(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel NormalSrcOver<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(NormalSrcOver(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel NormalSrcIn<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(NormalSrcIn(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel NormalSrcOut<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(NormalSrcOut(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel NormalDest<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(NormalDest(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel NormalDestAtop<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(NormalDestAtop(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel NormalDestOver<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(NormalDestOver(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel NormalDestIn<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(NormalDestIn(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel NormalDestOut<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(NormalDestOut(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel NormalClear<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(NormalClear(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel NormalXor<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(NormalXor(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }


        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 MultiplySrc(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return source;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 MultiplySrcAtop(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Atop(backdrop, source, Multiply(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 MultiplySrcOver(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Over(backdrop, source, Multiply(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 MultiplySrcIn(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return In(backdrop, source, Multiply(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 MultiplySrcOut(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Out(backdrop, source);
        }

       [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 MultiplyDest(Vector4 backdrop, Vector4 source, float opacity)
        {
            return backdrop;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 MultiplyDestAtop(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Atop(source, backdrop, Multiply(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 MultiplyDestOver(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Over(source, backdrop, Multiply(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 MultiplyDestIn(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return In(source, backdrop, Multiply(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 MultiplyDestOut(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Out(source, backdrop);
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 MultiplyXor(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Xor(backdrop, source);
        }   

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 MultiplyClear(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Clear(backdrop, source);
        }
        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel MultiplySrc<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(MultiplySrc(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel MultiplySrcAtop<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(MultiplySrcAtop(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel MultiplySrcOver<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(MultiplySrcOver(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel MultiplySrcIn<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(MultiplySrcIn(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel MultiplySrcOut<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(MultiplySrcOut(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel MultiplyDest<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(MultiplyDest(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel MultiplyDestAtop<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(MultiplyDestAtop(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel MultiplyDestOver<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(MultiplyDestOver(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel MultiplyDestIn<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(MultiplyDestIn(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel MultiplyDestOut<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(MultiplyDestOut(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel MultiplyClear<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(MultiplyClear(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel MultiplyXor<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(MultiplyXor(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }


        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 AddSrc(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return source;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 AddSrcAtop(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Atop(backdrop, source, Add(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 AddSrcOver(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Over(backdrop, source, Add(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 AddSrcIn(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return In(backdrop, source, Add(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 AddSrcOut(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Out(backdrop, source);
        }

       [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 AddDest(Vector4 backdrop, Vector4 source, float opacity)
        {
            return backdrop;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 AddDestAtop(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Atop(source, backdrop, Add(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 AddDestOver(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Over(source, backdrop, Add(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 AddDestIn(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return In(source, backdrop, Add(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 AddDestOut(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Out(source, backdrop);
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 AddXor(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Xor(backdrop, source);
        }   

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 AddClear(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Clear(backdrop, source);
        }
        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel AddSrc<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(AddSrc(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel AddSrcAtop<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(AddSrcAtop(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel AddSrcOver<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(AddSrcOver(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel AddSrcIn<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(AddSrcIn(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel AddSrcOut<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(AddSrcOut(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel AddDest<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(AddDest(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel AddDestAtop<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(AddDestAtop(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel AddDestOver<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(AddDestOver(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel AddDestIn<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(AddDestIn(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel AddDestOut<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(AddDestOut(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel AddClear<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(AddClear(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel AddXor<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(AddXor(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }


        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 SubtractSrc(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return source;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 SubtractSrcAtop(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Atop(backdrop, source, Subtract(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 SubtractSrcOver(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Over(backdrop, source, Subtract(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 SubtractSrcIn(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return In(backdrop, source, Subtract(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 SubtractSrcOut(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Out(backdrop, source);
        }

       [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 SubtractDest(Vector4 backdrop, Vector4 source, float opacity)
        {
            return backdrop;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 SubtractDestAtop(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Atop(source, backdrop, Subtract(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 SubtractDestOver(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Over(source, backdrop, Subtract(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 SubtractDestIn(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return In(source, backdrop, Subtract(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 SubtractDestOut(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Out(source, backdrop);
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 SubtractXor(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Xor(backdrop, source);
        }   

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 SubtractClear(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Clear(backdrop, source);
        }
        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel SubtractSrc<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(SubtractSrc(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel SubtractSrcAtop<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(SubtractSrcAtop(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel SubtractSrcOver<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(SubtractSrcOver(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel SubtractSrcIn<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(SubtractSrcIn(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel SubtractSrcOut<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(SubtractSrcOut(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel SubtractDest<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(SubtractDest(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel SubtractDestAtop<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(SubtractDestAtop(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel SubtractDestOver<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(SubtractDestOver(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel SubtractDestIn<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(SubtractDestIn(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel SubtractDestOut<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(SubtractDestOut(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel SubtractClear<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(SubtractClear(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel SubtractXor<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(SubtractXor(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }


        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 ScreenSrc(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return source;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 ScreenSrcAtop(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Atop(backdrop, source, Screen(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 ScreenSrcOver(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Over(backdrop, source, Screen(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 ScreenSrcIn(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return In(backdrop, source, Screen(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 ScreenSrcOut(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Out(backdrop, source);
        }

       [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 ScreenDest(Vector4 backdrop, Vector4 source, float opacity)
        {
            return backdrop;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 ScreenDestAtop(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Atop(source, backdrop, Screen(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 ScreenDestOver(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Over(source, backdrop, Screen(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 ScreenDestIn(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return In(source, backdrop, Screen(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 ScreenDestOut(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Out(source, backdrop);
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 ScreenXor(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Xor(backdrop, source);
        }   

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 ScreenClear(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Clear(backdrop, source);
        }
        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel ScreenSrc<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(ScreenSrc(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel ScreenSrcAtop<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(ScreenSrcAtop(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel ScreenSrcOver<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(ScreenSrcOver(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel ScreenSrcIn<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(ScreenSrcIn(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel ScreenSrcOut<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(ScreenSrcOut(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel ScreenDest<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(ScreenDest(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel ScreenDestAtop<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(ScreenDestAtop(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel ScreenDestOver<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(ScreenDestOver(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel ScreenDestIn<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(ScreenDestIn(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel ScreenDestOut<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(ScreenDestOut(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel ScreenClear<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(ScreenClear(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel ScreenXor<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(ScreenXor(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }


        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 DarkenSrc(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return source;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 DarkenSrcAtop(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Atop(backdrop, source, Darken(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 DarkenSrcOver(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Over(backdrop, source, Darken(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 DarkenSrcIn(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return In(backdrop, source, Darken(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 DarkenSrcOut(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Out(backdrop, source);
        }

       [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 DarkenDest(Vector4 backdrop, Vector4 source, float opacity)
        {
            return backdrop;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 DarkenDestAtop(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Atop(source, backdrop, Darken(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 DarkenDestOver(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Over(source, backdrop, Darken(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 DarkenDestIn(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return In(source, backdrop, Darken(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 DarkenDestOut(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Out(source, backdrop);
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 DarkenXor(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Xor(backdrop, source);
        }   

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 DarkenClear(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Clear(backdrop, source);
        }
        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel DarkenSrc<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(DarkenSrc(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel DarkenSrcAtop<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(DarkenSrcAtop(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel DarkenSrcOver<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(DarkenSrcOver(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel DarkenSrcIn<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(DarkenSrcIn(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel DarkenSrcOut<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(DarkenSrcOut(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel DarkenDest<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(DarkenDest(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel DarkenDestAtop<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(DarkenDestAtop(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel DarkenDestOver<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(DarkenDestOver(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel DarkenDestIn<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(DarkenDestIn(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel DarkenDestOut<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(DarkenDestOut(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel DarkenClear<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(DarkenClear(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel DarkenXor<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(DarkenXor(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }


        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 LightenSrc(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return source;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 LightenSrcAtop(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Atop(backdrop, source, Lighten(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 LightenSrcOver(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Over(backdrop, source, Lighten(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 LightenSrcIn(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return In(backdrop, source, Lighten(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 LightenSrcOut(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Out(backdrop, source);
        }

       [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 LightenDest(Vector4 backdrop, Vector4 source, float opacity)
        {
            return backdrop;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 LightenDestAtop(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Atop(source, backdrop, Lighten(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 LightenDestOver(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Over(source, backdrop, Lighten(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 LightenDestIn(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return In(source, backdrop, Lighten(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 LightenDestOut(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Out(source, backdrop);
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 LightenXor(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Xor(backdrop, source);
        }   

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 LightenClear(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Clear(backdrop, source);
        }
        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel LightenSrc<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(LightenSrc(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel LightenSrcAtop<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(LightenSrcAtop(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel LightenSrcOver<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(LightenSrcOver(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel LightenSrcIn<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(LightenSrcIn(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel LightenSrcOut<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(LightenSrcOut(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel LightenDest<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(LightenDest(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel LightenDestAtop<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(LightenDestAtop(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel LightenDestOver<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(LightenDestOver(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel LightenDestIn<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(LightenDestIn(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel LightenDestOut<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(LightenDestOut(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel LightenClear<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(LightenClear(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel LightenXor<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(LightenXor(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }


        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 OverlaySrc(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return source;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 OverlaySrcAtop(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Atop(backdrop, source, Overlay(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 OverlaySrcOver(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Over(backdrop, source, Overlay(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 OverlaySrcIn(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return In(backdrop, source, Overlay(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 OverlaySrcOut(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Out(backdrop, source);
        }

       [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 OverlayDest(Vector4 backdrop, Vector4 source, float opacity)
        {
            return backdrop;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 OverlayDestAtop(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Atop(source, backdrop, Overlay(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 OverlayDestOver(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Over(source, backdrop, Overlay(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 OverlayDestIn(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return In(source, backdrop, Overlay(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 OverlayDestOut(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Out(source, backdrop);
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 OverlayXor(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Xor(backdrop, source);
        }   

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 OverlayClear(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Clear(backdrop, source);
        }
        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel OverlaySrc<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(OverlaySrc(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel OverlaySrcAtop<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(OverlaySrcAtop(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel OverlaySrcOver<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(OverlaySrcOver(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel OverlaySrcIn<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(OverlaySrcIn(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel OverlaySrcOut<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(OverlaySrcOut(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel OverlayDest<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(OverlayDest(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel OverlayDestAtop<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(OverlayDestAtop(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel OverlayDestOver<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(OverlayDestOver(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel OverlayDestIn<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(OverlayDestIn(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel OverlayDestOut<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(OverlayDestOut(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel OverlayClear<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(OverlayClear(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel OverlayXor<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(OverlayXor(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }


        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 HardLightSrc(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return source;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 HardLightSrcAtop(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Atop(backdrop, source, HardLight(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 HardLightSrcOver(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Over(backdrop, source, HardLight(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 HardLightSrcIn(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return In(backdrop, source, HardLight(backdrop, source));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 HardLightSrcOut(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Out(backdrop, source);
        }

       [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 HardLightDest(Vector4 backdrop, Vector4 source, float opacity)
        {
            return backdrop;
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 HardLightDestAtop(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Atop(source, backdrop, HardLight(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 HardLightDestOver(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Over(source, backdrop, HardLight(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 HardLightDestIn(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return In(source, backdrop, HardLight(source, backdrop));
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 HardLightDestOut(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Out(source, backdrop);
        }

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 HardLightXor(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Xor(backdrop, source);
        }   

        [MethodImpl(MethodImplOptions.NoInlining)]
        public static Vector4 HardLightClear(Vector4 backdrop, Vector4 source, float opacity)
        {
            source.W *= opacity;

            return Clear(backdrop, source);
        }
        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel HardLightSrc<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(HardLightSrc(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel HardLightSrcAtop<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(HardLightSrcAtop(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel HardLightSrcOver<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(HardLightSrcOver(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel HardLightSrcIn<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(HardLightSrcIn(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel HardLightSrcOut<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(HardLightSrcOut(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel HardLightDest<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(HardLightDest(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel HardLightDestAtop<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(HardLightDestAtop(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel HardLightDestOver<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(HardLightDestOver(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel HardLightDestIn<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(HardLightDestIn(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel HardLightDestOut<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(HardLightDestOut(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel HardLightClear<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(HardLightClear(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

        

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public static TPixel HardLightXor<TPixel>(TPixel backdrop, TPixel source, float opacity)
            where TPixel : struct, IPixel<TPixel>
        {
            opacity = opacity.Clamp(0, 1);
            TPixel dest = default;
            dest.FromScaledVector4(HardLightXor(backdrop.ToScaledVector4(), source.ToScaledVector4(), opacity));
            return dest;
        }

    }
}